package domain;

import junit.framework.*;

public class GameTest extends TestCase {
   private static final int BURN = 1;
   private Game game;
   private Deck deck;
   private Player player1;
   private Player player2;
   private Player player3;
   private static final int STAKE = 1000;
   private static final int SMALL = 10;
   private static final int BIG = 20;

   protected void setUp() {
      game = new Game();
      game.setBlinds(SMALL, BIG);

      deck = game.deck();

      player1 = new Player("a");
      player1.add(STAKE);
      player2 = new Player("b");
      player2.add(STAKE);
      player3 = new Player("c");
      player3.add(STAKE);
   }

   public void testCreate() {
      assertPlayers();
   }

   public void testAddSinglePlayer() {
      final String name = "Jeff";
      game.add(new Player(name));
      assertPlayers(name);
   }

   public void testAddMaximumNumberOfPlayers() {
      for (int i = 0; i < Game.CAPACITY; i++)
         game.add(new Player("" + i));
      assertPlayers("0", "1", "2", "3", "4", "5", "6", "7", "8", "9");
   }

   public void testDealCompleteHand() {
      addTwoPlayers();
      game.setButton(2);
      game.startHand();

      Card[] hole = deck.top(4);
      game.dealHoleCards();

      assertHoleCards(player1, hole, 0, 2);
      assertHoleCards(player2, hole, 1, 3);

      int remaining = Deck.SIZE - hole.length;
      assertEquals(remaining, deck.cardsRemaining());
      Card[] flop = deck.top(BURN + 3);

      game.dealFlop();
      remaining -= flop.length;
      assertCardsDealt(remaining, flop);
      CardTest.assertCards(game.community(), flop[1], flop[2], flop[3]);

      Card[] turn = deck.top(BURN + 1);
      game.dealTurn();
      remaining -= turn.length;
      assertCardsDealt(remaining, turn);
      CardTest
            .assertCards(game.community(), flop[1], flop[2], flop[3], turn[1]);

      Card[] river = deck.top(BURN + 1);
      game.dealRiver();
      remaining -= river.length;
      assertCardsDealt(remaining, river);
      CardTest.assertCards(game.community(), flop[1], flop[2], flop[3],
            turn[1], river[1]);
   }
   
   public void testDealOrderStartsFromButton() {
      addTwoPlayers();
      game.setButton(1);
      game.startHand();
      
      Card[] hole = deck.top(4);
      dealAllCardsInHand();

      assertHoleCards(player1, hole, 1, 3);
      assertHoleCards(player2, hole, 0, 2);
      
      game.stopHand();
      
      game.startHand();
      hole = deck.top(4);
      dealAllCardsInHand();

      assertHoleCards(player1, hole, 0, 2);
      assertHoleCards(player2, hole, 1, 3);
   }

   public void testBlinds() {
      addThreePlayers();
      game.setButton(3);

      game.startHand();

      assertEquals(STAKE - SMALL, player1.chipCount());
      assertEquals(STAKE - BIG, player2.chipCount());
      assertEquals(STAKE, player3.chipCount());
   }

   public void testHandFlow() {
      addThreePlayers();
      game.setButton(3);

      game.startHand();
      dealAllCardsInHand();
      
      game.stopHand();
      assertEquals(1, game.buttonPosition());
      assertNoCardsOut();

      game.startHand();
      dealAllCardsInHand();
      game.stopHand();
      assertEquals(2, game.buttonPosition());
      assertNoCardsOut();
      
      game.startHand();
      dealAllCardsInHand();
      game.stopHand();
      assertEquals(3, game.buttonPosition());
      assertNoCardsOut();
      
      fail("need to ensure blinds are extracted properly");
   }
   
   // missing tests:
   //  - use a new deck each time!

   private void assertNoCardsOut() {
      for (Player player: game.players())
         assertTrue(player.holeCards().isEmpty());
      assertTrue(game.community().isEmpty());
   }

   private void dealAllCardsInHand() {
      game.dealHoleCards();
      game.dealFlop();
      game.dealTurn();
      game.dealRiver();
   }

   private void addThreePlayers() {
      game.add(player1);
      game.add(player2);
      game.add(player3);
   }

   private void addTwoPlayers() {
      game.add(player1);
      game.add(player2);
   }

   private void assertCardsDealt(int remaining, Card[] turn) {
      assertDeckCount(remaining);
      DeckTest.assertCardsDealt(deck, turn);
   }

   private void assertHoleCards(Player player, Card[] hole, int... indices) {
      Card[] cards = new Card[indices.length];
      for (int i = 0; i < indices.length; i++)
         cards[i] = hole[indices[i]];
      DeckTest.assertCardsDealt(deck, cards);
      CardTest.assertCards(player.holeCards(), cards);
   }

   private void assertDeckCount(int expected) {
      assertEquals(expected, deck.cardsRemaining());
   }

   private void assertPlayers(String... expected) {
      assertEquals(expected.length, game.players().size());
      int i = 0;
      for (Player player : game.players())
         assertEquals(expected[i++], player.getName());
   }
}